/* **
  build with: g++ -Wall -Zexe -Zomf allocmem.c allocmem.def
** */

#define MODERN
#define INCL_BASE
#define INCL_DOS
#include <os2.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <process.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#define PAGESIZE 4096L

int   i;
ULONG n;

void (* sig_segv_old)(int);
void sig_segv_handler (int signum)
{
    fprintf(stderr, "?Segment violation, out of range memory access, bad stuff like that.\n");
    fprintf(stderr, " Occurred when the memory value became %ld on the %dth page\n", n, i);
    exit (signum);
}


int main (int argc, char **argv)
{
    void * baseptr;
    char * ptr;
    char * bptr;
    ULONG n1;
    ULONG physmem_resmem[2];
    APIRET rc;


    sig_segv_old = signal(SIGSEGV, &sig_segv_handler);

    rc = DosQuerySysInfo(QSV_TOTPHYSMEM, QSV_TOTRESMEM, &physmem_resmem,
sizeof(physmem_resmem));
    switch (rc)
    {
	default:
	case   0: break;
	case  87: fprintf(stderr, "Invalid Parameter\n"); break;
	case 111: fprintf(stderr, "Buffer Overflow\n"); break;
    }
    if (0 != rc)
	exit(0);

    if (argc != 2)
	n = physmem_resmem[0] - physmem_resmem[1];
    else
    {
	/* round up to 4K page */
	n = (1024L * atoi(argv[1]) + 4095L) & 0xFFFFF000;
	if (0 == n)
	{
	    printf("?[%d] is an invalid value\n", atoi(argv[1]));
	    puts("Usage: allocmem [# of K]\n" );
	    exit(1);
	}
    }

    printf("Physical Memory %ld K,  Resident Memory %ld K\n",
	    physmem_resmem[0] / 1024L, physmem_resmem[1] / 1024L);

    rc = DosAllocMem(&baseptr, (n), PAG_COMMIT | PAG_WRITE | OBJ_ANY);

    /* Hunt for an amount to allocate that is legal. */
    if (8 == rc)
    {
	// If not enough memory, decrease allocation until there is. Binary search.
	n1 = n;
	do {
	    n1 = n1 / 2;
	    if (8 == rc)
	    {
		n -=  n1;
		rc = DosAllocMem(&baseptr, (n), PAG_READ | OBJ_ANY);
	     }
	    else	// 0 == rc
	    {
		n +=  n1;
		rc = DosFreeMem(baseptr);
		rc = DosAllocMem(&baseptr, (n), PAG_READ | OBJ_ANY);
	     }
	 } while (PAGESIZE < n1);
	
	if (0 == rc)
	    rc = DosFreeMem(baseptr);
	else
            n -= n1;
	rc = DosAllocMem(&baseptr, (n), PAG_COMMIT | PAG_WRITE | OBJ_ANY);
     }
    printf("Allocating %ld K... ", n / 1024L);

    switch (rc)
    {
	default:
	case  0: break;
	case  8: fprintf(stderr, "Not Enough Memory\n"); break;
	case 87: fprintf(stderr, "Invalid Parameter\n"); break;
	case 95: fprintf(stderr, "Interrupted system call\n"); break;
    }
    if (0 != rc)
	exit(0);

    bptr = (char *) baseptr;
    ptr  = bptr + (n - PAGESIZE);
    i = 0;
    do
    {
	++i;
	
	*ptr = 0;		/* Force it to allocate		*/
	ptr -= PAGESIZE;    	/* Move to the next page.	*/
	DosSleep(0);
    }
    while (bptr <= ptr);

    printf("\n");
    DosSleep(10000);

    exit (0);
}

